/*
 * Filename: NSGlobal.h
 * Project:  Pascal's X Kit
 * Module:   Basic application components
 *
 * Function: This header file contains the global definitions,
 *           types and constants
 *
 * Author:   Pascal Forget <pascal@wsc.com>
 * Date:     January 1996
 *
 * Copyright (C) 1996 Pascal Forget. All Rights Reserved.
 */

#ifndef NSGLOBAL_H__
#define NSGLOBAL_H__

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <X11/cursorfont.h>
#include <math.h>
#include <objc/Object.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <unistd.h>
#include <X11/keysymdef.h>

#include <Foundation/NSArray.h>
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSDate.h>
#include <Foundation/NSString.h>

/* Some useful #defines */

#define NSX_MAX(a,b) (a>b)?a:b

/* Not part of Types and constants */

typedef double NSCoord;

typedef struct {
    NSCoord x;
    NSCoord y;
} NSPoint;

typedef struct {
    NSCoord height;
    NSCoord width;
} NSSize;

typedef struct {
    NSPoint origin;
    NSSize size;
} NSRect;


Display *NSDisplay;
int NSScreen;
GC NSBlackGC;
GC NSWhiteGC;
GC NSLightgrayGC;
GC NSDarkgrayGC;

/* END Not part of Types and constants */


/* Application */

id NSApp;     /* Represents the application's NSApplication object */

/*
 * The following structure stores information used by the system
 * during a modal session
 */
typedef struct _NSModalSession *NSModalSession;

enum {
    NSRunStoppedResponse,
    NSRunAbortedResponse,
    NSRunContinuesResponse
};

NSString *NSModalPanelRunLoopMode; /* Input-filter modes passed to NSRunLoop */

NSString *NSEventTrackingRunLoopMode;

/* Box */

typedef enum _NSTitlePosition {    /* This type's constants represent the    */
    NSNoTitle,                     /* locations where an NSBox's title is    */
    NSAboveTop,                    /* placed in relation to the border       */
    NSAtTop,                       /* (setTitlePosition: and titlePosition)  */
    NSBelowTop,
    NSAboveBottom,
    NSAtBottom,
    NSBelowBottom
} NSTitlePosition;

/* Buttons */

typedef enum _NSButtonType {       /* The constants of NSButtonType indicate */
    NSMomentaryPushButton,         /* the way NSButtons and NSButtonCells    */
    NSPushOnPushOffButton,         /* behave when pressed, and how they      */
    NSToggleButton,                /* display their state.  They are used in */
    NSSwitchButton,                /* NSButton's setType: method.            */
    NSRadioButton,
    NSMomentaryChangeButton,
    NSOnOffButton
} NSButtonType;

/* Cells and ButtonCells */

typedef enum _NSCellType {         /* Represent different types of NSCells   */
    NSNullCellType,                /* No display.                            */
    NSTextCellType,                /* Displays text.                         */
    NSImageCellType                /* Displays an image.                     */
} NSCellType;                      /* Returned from type and setType:        */

typedef enum _NSCellImagePosition { /* The position of the image relative to */
    NSNoImage,                      /* the cell's title.  Returned from      */
    NSImageOnly,                    /* imagePosition and set through         */
    NSImageLeft,                    /* setImagePosition:.                    */
    NSImageRight,
    NSImageBelow,
    NSImageAbove,
    NSImageOverlaps
} NSCellImagePosition;

typedef enum _NSCellAttribute {    /* The constant values of NSCellAttribute */
    NSCellDisabled,                /* represent parameters that you can set  */
    NSCellState,                   /* and access through NSCell's and        */
    NSPushInCell,                  /* NSButtonCell's setParameter:to: and    */
    NSCellEditable,                /* getParameter: methods.  Only the first */
    NSChangeGrayCell,              /* five constants are used by NSCell; the */
    NSCellHighlighted,             /* others apply to NSButtonCells only.    */
    NSCellLightsByContents,
    NSCellLightsByGray,
    NSChangeBackgroundCell,
    NSCellLightsByBackground,
    NSCellIsBordered,
    NSCellHasOverlappingImage,
    NSCellHasImageHorizontal,
    NSCellHasImageOnLeftOrBottom,
    NSCellChangesContents,
    NSCellIsInsetButton
} NSCellAttribute;

enum {                             /* Numeric data types that an NSCell can  */
    NSAnyType,                     /* accept. Used as the argument for       */
    NSIntType,                     /* setEntryType:.                         */
    NSPositiveIntType,
    NSFloatType,
    NSPositiveFloatType,
    NSDateType,
    NSDoubleType,
    NSPositiveDoubleType
};

enum {                             /* NSButtonCell uses these values to      */
    NSNoCellMask,                  /* determine how to highlight a button    */
    NSContentsCellMask,            /* cell or show an ON state.              */
    NSPushInCellMask,              /* (returned/passed in showStateBy/       */
    NSChangeGrayCellMask,          /* setShowStateBy: and highlightsBy/      */
    NSChangeBackgroundCellMask     /* setHighlightsBy:).                     */
};

/* Color */

enum {                             /* Tags that identify modes (or views) in */
    NSGrayModeColorPanel,          /* the color panel.                       */
    NSRGBModeColorPanel,
    NSCMYKModeColorPanel,
    NSHSBModeColorPanel,
    NSCustomPaletteModeColorPanel,
    NSColorListModeColorPanel,
    NSWheelModeColorPanel
};

enum {                             /* Bit masks for determining the current  */
    NSColorPanelGrayModeMask,      /* mode (or view) of the color panel.     */
    NSColorPanelRGBModeMask,
    NSColorPanelCMYKModeMask,
    NSColorPanelHSBModeMask,
    NSColorPanelCustomPaletteModeMask,
    NSColorPanelColorListModeMask,
    NSColorPanelWheelModeMask,
    NSColorPanelAllModesMask
};

/* Data Link */

typedef int NSDataLinkNumber;      /* Returned by NSDataLink's linkNumber    */
                                   /* method as a persistent identifier of a */
                                   /* destination link                       */

NSString  *NSDataLinkFileNameExtension;  /* The filename suffix to be used   */
                                         /* when data links are saved. The   */
                                         /* default is 'objlink'.            */

typedef enum _NSDataLinkDisposition {  /* Returned by NSDataLink's disposi-  */
    NSLinkInDestination,               /* tion method to identify a link as  */
    NSLinkInSource,                    /* a destination link, a source link, */
    NSLinkBroken                       /* or a broken link.                  */
} NSDataLinkDisposition;

typedef enum _NSDataLinkUpdateMode {   /* Identifies when a link's data is   */
    NSUpdateContinuously,              /* to be updated.  Set through the    */
    NSUpdateWhenSourceSaved,           /* setUpfdateMode: method and returned*/
    NSUpdateManually,                  /* by updateMode.                     */
    NSUpdateNever
} NSDataLinkUpdateMode;


/* Drag Operation */

typedef enum _NSDragOperation {    /* The constants of this type identify    */
    NSDragOperationNone,           /* different kinds of dragging operations.*/
    NSDragOperationCopy,           /* NSDragOperationNone implies that the   */
    NSDragOperationLink,           /* operation is rejected.                 */
    NSDragOperationGeneric,
    NSDragOperationPrivate,        /* NSDragOperationPrivate means that the  */
    NSDragOperationAll             /* system leaves the cursor alone.        */
} NSDragOperation;


/* Event Handling */

typedef enum _NSEventType {        /* Each constant of NSEventType identifies*/
    NSLeftMouseDown,               /* an event type. See NSEvent class.      */
    NSLeftMouseUp,
    NSRightMouseDown,
    NSRightMouseUp,
    NSMouseMoved,
    NSLeftMouseDragged,
    NSRightMouseDragged,
    NSMouseEntered,
    NSMouseExited,
    NSKeyDown,
    NSKeyUp,
    NSFlagsChanged,
    NSPeriodic,
    NSCursorUpdate
} NSEventType;

enum {                                 /* Unicodes identifying function keys */
    NSUpArrowFunctionKey = 0xF700,     /* on the keyboard, OpenStep reserves */
    NSDownArrowFunctionKey = 0xF701,   /* the range 0xF700-0xF8FF for this   */
    NSLeftArrowFunctionKey = 0xF702,   /* purpose.  The availability of some */
    NSRightArrowFunctionKey = 0xF703,  /* keys is system-dependent.          */
    NSF1FunctionKey = 0xF704,
    NSF2FunctionKey = 0xF705,
    NSF3FunctionKey = 0xF706,
    NSF4FunctionKey = 0xF707,
    NSF5FunctionKey = 0xF708,
    NSF6FunctionKey = 0xF709,
    NSF7FunctionKey = 0xF70A,
    NSF8FunctionKey = 0xF70B,
    NSF9FunctionKey = 0xF70C,
    NSF10FunctionKey = 0xF70D,
    NSF11FunctionKey = 0xF70E,
    NSF12FunctionKey = 0xF70F,
    NSF13FunctionKey = 0xF710,
    NSF14FunctionKey = 0xF711,
    NSF15FunctionKey = 0xF712,
    NSF16FunctionKey = 0xF713,
    NSF17FunctionKey = 0xF714,
    NSF18FunctionKey = 0xF715,
    NSF19FunctionKey = 0xF716,
    NSF20FunctionKey = 0xF717,
    NSF21FunctionKey = 0xF718,
    NSF22FunctionKey = 0xF719,
    NSF23FunctionKey = 0xF71A,
    NSF24FunctionKey = 0xF71B,
    NSF25FunctionKey = 0xF71C,
    NSF26FunctionKey = 0xF71D,
    NSF27FunctionKey = 0xF71E,
    NSF28FunctionKey = 0xF71F,
    NSF29FunctionKey = 0xF720,
    NSF30FunctionKey = 0xF721,
    NSF31FunctionKey = 0xF722,
    NSF32FunctionKey = 0xF723,
    NSF33FunctionKey = 0xF724,
    NSF34FunctionKey = 0xF725,
    NSF35FunctionKey = 0xF726,
    NSInsertFunctionKey = 0xF727,
    NSDeleteFunctionKey = 0xF728,
    NSHomeFunctionKey = 0xF729,
    NSBeginFunctionKey = 0xF72A,
    NSEndFunctionKey = 0xF72B,
    NSPageUpFunctionKey = 0xF72C,
    NSPageDownFunctionKey = 0xF72D,
    NSPrintScreenFunctionKey = 0xF72E,
    NSScrollLockFunctionKey = 0xF72F,
    NSPauseFunctionKey = 0xF730,
    NSSysReqFunctionKey = 0xF731,
    NSBreakFunctionKey = 0xF732,
    NSResetFunctionKey = 0xF733,
    NSStopFunctionKey = 0xF734,
    NSMenuFunctionKey = 0xF735,
    NSUserFunctionKey = 0xF736,
    NSSystemFunctionKey = 0xF737,
    NSPrintFunctionKey = 0xF738,
    NSClearLineFunctionKey = 0xF739,
    NSClearDisplayFunctionKey = 0xF73A,
    NSInsertLineFunctionKey = 0xF73B,
    NSDeleteLineFunctionKey = 0xF73C,
    NSInsertCharFunctionKey = 0xF73D,
    NSDeleteCharFunctionKey = 0xF73E,
    NSPrevFunctionKey = 0xF73F,
    NSNextFunctionKey = 0xF740,
    NSSelectFunctionKey = 0xF741,
    NSExecuteFunctionKey = 0xF742,
    NSUndoFunctionKey = 0xF743,
    NSRedoFunctionKey = 0xF744,
    NSFindFunctionKey = 0xF745,
    NSHelpFunctionKey = 0xF746,
    NSModeSwitchFunctionKey = 0xF747
};

enum {                             /* Device-independent bit masks for       */
    NSAlphaShiftKeyMask,           /* evaluating event-modifier flags to     */
    NSShiftKeyMask,                /* determine which modifier key (if any)  */
    NSControlKeyMask,              /* was pressed.                           */
    NSAlternateKeyMask,
    NSCommandKeyMask,
    NSNumericPadKeyMask,
    NSHelpKeyMask,
    NSFunctionKeyMask
};

enum {                             /* Bit masks for determining the event    */
    NSLeftMouseDownMask,           /* type.                                  */
    NSLeftMouseUpMask,
    NSRightMouseDownMask,
    NSRightMouseUpMask,
    NSMouseMovedMask,
    NSLeftMouseDraggedMask,
    NSRightMouseDraggedMask,
    NSMouseEnteredMask,
    NSMouseExitedMask,
    NSKeyDownMask,
    NSKeyUpMask,
    NSFlagsChangedMask,
    NSPeriodicMask,
    NSCursorUpdateMask,
    NSAnyEventMask
};


/* Exceptions */

/* Global Exception Strings

   The following global strings identify the exceptions returned by
   various operations in the Application Kit.  They are defined in
   NSError.h
*/

#include "NSErrors.h"


/* Fonts */

typedef unsigned int NSFontTraitMask;  /* Characterizes one or more font's   */
				       /* traits.  It is used as an argument */
                                       /* type for several of the methods in */
                                       /* the NSFontManager class.           */
                                       /* You build a mask by OR'ing together*/
                                       /* the following enumeration constants*/

enum {                                 /* Values used by NSFontManager to    */
    NSItalicFontMask,                  /* identify font traits.              */
    NSBoldFontMask,
    NSUnboldFontMask,
    NSNonStandardCharacterSetFontMask,
    NSNarrowFontMask,
    NSExpandedFontMask,
    NSCondensedFontMask,
    NSSmallCapsFontMask,
    NSPosterFontMask,
    NSCompressedFontMask,
    NSUnitalicFontMask
};

typedef unsigned int NSGlyph;      /* A type for numbers identifying font    */
                                   /* glyphs.  It is used as the argument    */
                                   /* type for several of the methods in the */
                                   /* NSFont class.                          */



enum {                           /* Tags identifying views in the font panel */
    NSFPPreviewButton,
    NSFPRevertButton,
    NSFPSetButton,
    NSFPPreviewField,
    NSFPSizeField,
    NSFPSizeTitle,
    NSFPCurrentField
};

const float *NSFontIdentityMatrix;  /* Identifies a font matrix that is used */
                                    /* for fonts displayed in an NSView      */
                                    /* having an unflipped coordinate system.*/
                                  

NSString *NSAFMAscender;           /* Global keys to access the values avai- */
NSString *NSAFMCapHeight;          /* lable in the AFM dictionary.  You can  */
NSString *NSAFMCharacterSet;       /* convert the appropriate values (e.g.   */
NSString *NSAFMDescender;          /* ascender,cap,height) to floating point */
NSString *NSAFMEncodingScheme;     /* values by using NSString's floatValue  */
NSString *NSAFMFamilyName;         /* method.                                */
NSString *NSAFMFontName;
NSString *NSAFMFormatVersion;
NSString *NSAFMFullName;
NSString *NSAFMItalicAngle;
NSString *NSAFMMappingScheme;
NSString *NSAFMNotice;
NSString *NSAFMUnderlinePosition;
NSString *NSAFMUnderlineThickness;
NSString *NSAFMFMVersion;
NSString *NSAFMFMWeight;
NSString *NSAFMXHeight;


/* Graphics */

typedef int NSWindowDepth;         /* This type gives the window depth limit.*/
                                   /* Use NSAvailableWindowDepth(),          */
                                   /* NSBitsPerPixelFromDepth(),             */
                                   /* NSPlanarFromDepth(), and               */
                                   /* NSColorSpaceFromDepth() to extract     */
                                   /* information from a window depth.  The  */
                                   /* NSWindowDepth type is also used as an  */
                                   /* argument type of methods in NSScreen   */
                                   /* and NSWindow.                          */


typedef enum _NSTIFFCompression {
    NSTIFFCompressionNone = 1,
    NSTIFFCompressionCCITTFAX3 = 3,
    NSTIFFCompressionCCITTFAX4 = 4,
    NSTIFFCompressionLZW = 5,
    NSTIFFCompressionJPEG = 6,
    NSTIFFCompressionNEXT = 32766,
    NSTIFFCompressionPackBits = 32773,
    NSTIFFCompressionOldJPEG = 32865
} NSTIFFCompression;

enum {
    NSImageRepMatchesDevice
};

/* Colorspace names */

/* Predefined colorspace names.  Used as arguments in NSDrawBitMap() and     */
/* NSNumberOfColorComponents(); value returned from NSColorSpaceFromDepth(). */

NSString *NSCalibratedWhiteColorSpace;

NSString *NSCalibratedBlackColorSpace;

NSString *NSCalibratedRGBColorSpace;

NSString *NSDeviceWhiteColorSpace;

NSString *NSDeviceBlackColorSpace;

NSString *NSDeviceRGBColorSpace;

NSString *NSDeviceCMYKColorSpace;

NSString *NSNamedColorSpace;

NSString *NSCustomColorSpace;


/* Gray Values */

const float NSBlack;                   /* Standard gray values for the 2 bit */
const float NSDarkGray;                /* grayscale colorspace               */
const float NSWhite;
const float NSLightGray;


/* Device Dictionary Keys */

/* Keys to get designated values from device dictionaries */

NSString *NSDeviceResolution;

NSString *NSDeviceColorSpaceName;

NSString *NSDeviceBitsPerSample;

NSString *NSDeviceIsScreen;

NSString *NSDeviceIsPrinter;

NSString *NSDeviceSize;


/* Matrix */

typedef enum _NSMatrixMode {       /* The constants in this type represent   */
    NSRadioModeMatrix,             /* the modes of operation of an NSMatrix  */
    NSHighlightModeMatrix,
    NSListModeMatrix,
    NSTrackModeMatrix
} NSMatrixMode;


/* !@#$%^&*()_+!@#$%^&*()_+!@#$%^&*()_+!@#$%^&*()_+!@#$%^&*()_+!@#$%^&*()_+! */

/* most Notifications are missing below... */

NSString *NSViewFrameChangedNotification;       /* NSView */

/* !@#$%^&*()_+!@#$%^&*()_+!@#$%^&*()_+!@#$%^&*()_+!@#$%^&*()_+!@#$%^&*()_+! */


/* Panel */

enum {                             /* Values returned by the standard panel  */
    NSOKButton = 1,                /* buttons, OK and Cancel.                */
    NSCancelButton = 0
};

enum {
    NSAlertDefaultReturn = 1,      /* Values returned by NSRunAlertPanel()   */
    NSAlertAlternateReturn = 0,    /* function and by runModalSession: when  */
    NSAlertOtherReturn = -1,       /* the modal session is run with a Panel  */
    NSAlertErrorReturn = -2        /* provided by NSGetAlertPanel().         */
};


/* Page Layout */

enum {                             /* Tags identifying buttons, fields and   */
    NSPLImageButton,               /* other views of the Page Layout panel.  */
    NSPLTitleField,
    NSPLPaperNameButton,
    NSPLUnitsButton,
    NSPLWidthForm,
    NSPLHeightForm,
    NSPLOrientationMatrix,
    NSPLCancelButton,
    NSPLOKButton
};

/* Pasteboard stuff missing HERE */


/* Printing */

typedef enum _NSPrinterTableStatus {
    NSPrinterTableOK,
    NSPrinterTableNotFound,
    NSPrinterTableError
} NSPrinterTableStatus;

typedef enum _NSPrintingOrientation {
    NSPortraitOrientation,
    NSLandscapeOrientation
} NSPrintingOrientation;

typedef enum _NSPrintingPageOrder {
    NSDescendingPageOrder,
    NSSpecialPageOrder,
    NSAscendingPageOrder,
    NSUnknownPageOrder
} NSPrintingPageOrder;

typedef enum _NSPrintingPaginationMode {
    NSAutoPagination,
    NSFitPagination,
    NSClipPagination
} NSPrintingPaginationMode;


/* Save Panel */

enum {                               /* Tags identifying buttons, fields and */
    NSFileHandlingPanelImageButton,  /* other views in the Save Panel        */
    NSFileHandlingPanelTitledField,
    NSFileHandlingPanelBrowser,
    NSFileHandlingPanelCancelButton,
    NSFileHandlingPanelOKButton,
    NSFileHandlingPanelForm,
    NSFileHandlingPanelHomeButton,
    NSFileHandlingPanelDiskButton,
    NSFileHandlingPanelDiskEjectButton
};    


/* Scroller */

typedef enum _NSScrollArrowPosition {  /* NSScroller uses these constants in */
    NSScrollerArrowsMaxEnd,            /* its setArrowPosition: method to    */
    NSScrollerArrowsMinEnd,            /* set the position of the arrows     */
    NSScrollerArrowsNone               /* within the scroller                */
} NSScrollArrowPosition;

typedef enum _NSScrollerPart {         /* NSScroller uses these constants in */
    NSScrollerNoPart,                  /* its hitPart method to identify the */
    NSScrollerDecrementPage,           /* part of the scroller specified in  */
    NSScrollerKnob,                    /* a mouse event                      */
    NSScrollerIncrementPage,
    NSScrollerDecrementLine,
    NSScrollerIncrementLine,
    NSScrollerKnobSlot
} NSScrollerPart;

typedef enum _NSScrollerUsablePart {   /* The usable parts of a scroller     */
    NSNoScrollerParts,
    NSOnlyScrollerArrows,
    NSAllScrollerParts
} NSUsableScrollerParts;

typedef enum _NSScrollerArrow {        /* These are the two types of arrows. */
    NSScrollerIncrementArrow,          /* NSScroller's drawArrow:highlight:  */
    NSScrollerDecrementArrow           /* method takes an NSScrollerArrow as */
} NSScrollerArrow;                     /* the first argument.                */

const float NSScrollerWidth;           /* Identifies the default width of a  */
                                       /* vertical scroller and the default  */
                                       /* height of an horizontal scroller.  */


/* Text stuff not implemented yet */


/* View */

typedef int NSTrackingRectTag;     /* A unique identifier of a tracking      */
                                   /* rectangle assigned by NSView. (See     */
                                   /* addTrackingRectangle:owner:assumeInsi- */
                                   /* de:)                                   */

typedef enum _NSBorderType {       /* The 4 types of borders that can appear */
    NSNoBorder,                    /* around NSView objects                  */
    NSLineBorder,
    NSBezelBorder,
    NSGrooveBorder
} NSBorderType;

enum {                              /* NSView uses these autoresize constants*/
    NSViewNotSizable,               /* to describe the parts of a view (or   */
    NSViewMinXMargin,               /* its margins) that are resized when    */
    NSViewWidthSizable,             /* the view's superview is resized.      */
    NSViewMaxXMargin,
    NSViewMinYMargin,
    NSViewHeightSizable,
    NSViewMaxYMargin
};


/* Window */

enum {                              /* These constants list the window-device*/
    NSNormalWindowLevel = 0,        /* tiers that the Application Kit uses.  */
    NSFloatingWindowLevel = 3,      /* Windows are ordered (or "layered")    */
    NSDockWindowLevel = 5,          /* within tiers: The uppermost window in */
    NSSubmenuWindowLevel = 10,      /* one tier can still be obscured by the */
    NSMainMenuWindowLevel = 20      /* lowest window in the next higher tier.*/
};

enum {                              /* Bitmap masks to determine certain     */
    NSBorderlessWindowMask,         /* window styles.                        */
    NSTitledWindowMask,
    NSClosableWindowMask,
    NSMiniuaturizableWindowMask,
    NSResizableWindowMask
};

/* Size Globals.  These global constants give the dimensions of an icon and  */
/* contained (sic)                                                           */

NSSize NSIconSize;

NSSize NSTokenSize;


/* Workspace */

/* Workspace File Type Globals  */

/* Identifies the type of file queried by the method                         */
/* getInfoForFile:application:type: (passed back by reference in last arg).  */

NSString *NSPlainFileType;

NSString *NSDirectoryFileType;

NSString *NSApplicationFileType;

NSString *NSFilesystemFileType;

NSString *NSShellCommandFileType;


/* Workspace File Operation Globals */

/* Used as file-operation arguments in the                                   */
/* performFileOperation:source:destination:files:options: method (1st arg.). */

NSString *NSWorkspaceCompressOperation;

NSString *NSWorkspaceCopyOperation;

NSString *NSWorkspaceDecompressOperation;

NSString *NSWorkspaceDecryptOperation;

NSString *NSWorkspaceDestroyOperation;

NSString *NSWorkspaceDuplicateOperation;

NSString *NSWorkspaceEncryptOperation;

NSString *NSWorkspaceLinkOperation;

NSString *NSWorkspaceMoveOperation;

NSString *NSWorkspaceRecycleOperation;

typedef enum _NSTextAlignment {
    NSLeftTextAlignment,
    NSRightTextAlignment,
    NSCenterTextAlignment,
    NSJustifiedTextAlignment,
    NSNaturalTextAlignment
} NSTextAlignment;

#endif
