/* 
   PXKScroller.h

   NSScroller for GNUstep GUI X/DPS Backend

   A scroller is used for controlling the visible portion
   of another view, such as a selection list or a large
   image.

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Pascal Forget <pascal@wsc.com>
   Date: October 1995
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#ifndef _GNUstep_H_PXKScroller
#define _GNUstep_H_PXKScroller

#include <gnustep/xdps/stdpxk.h>
#include <AppKit/NSScroller.h>
#include <gnustep/xdps/PXKScrollButton.h>

#define PXK_ARROW_SIZE 16

/*
 * The scrolling button and possibly the two arrow buttons
 * are stored in the "subviews" instance variable (PXKView)
 * at positions 0, 1 and 2.
 */

@interface PXKScroller : NSScroller
{
  /* These need to be moved to backend structure */
    float maxValue;
    float minValue;
    float currentValue;
    float ratio;
    float step;
    BOOL showArrows;
    BOOL vertical;
}

- (BOOL)arrowsAreVisible;
- (float)currentValue;
- decrement;
- drawBorder;
- hideArrows;
- increment;
- (float)maxValue;
- (float)minValue;
- placeSubviews;
- (float)ratio;
- (float)relativePosition;
- setCurrentValue:(float)value;
- setMaxValue:(float)value;
- setMinValue:(float)value;
- setRatio:(float)r;
- setStep:(float)s;
- showArrows;
- (float)step;
- (BOOL)vertical;

@end

#endif /* _GNUstep_H_PXKScroller */
