/* 
   PXKColor.m

   NSColor for GNUstep GUI X/DPS Backend

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Pascal Forget <pascal@wsc.com>
   Date: January 1996
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include <gnustep/xdps/PXKColor.h>
#include <gnustep/xdps/PXKDPSContext.h>
#include <gnustep/xdps/PXKScreen.h>

//
// Backend structure for PXKColor
//
typedef struct _PXKColor_struct
{
  XColor color;
  BOOL is_alloced;
} PXKColor_struct;

#define PXKCOLOR (((PXKColor_struct *)be_color_reserved)->color)
#define PXKALLOC (((PXKColor_struct *)be_color_reserved)->is_alloced)

@implementation PXKColor (GNUstepXDPS)

- (XColor)xColor
{
  // If we haven't allocated it yet
  if (!PXKALLOC)
    {
      Display *xdis = [(PXKDPSContext *)[NSDPSContext currentContext] 
					xDisplay];
      int xscreen = [(PXKScreen *)[NSScreen mainScreen] xScreen];
      Status ret;

      // Copy values from the NSColor instance variables
      PXKCOLOR.red = (unsigned short)(65535 * RGB_component.red);
      PXKCOLOR.green = (unsigned short)(65535 * RGB_component.green);
      PXKCOLOR.blue = (unsigned short)(65535 * RGB_component.blue);

      // Allocate the color
      ret = XAllocColor(xdis, XDefaultColormap(xdis, xscreen), &PXKCOLOR);
      if (!ret)
	NSLog(@"Failed to allocate color\n");

      // Copy the actual values back to the NSColor variables
      RGB_component.red = ((float)PXKCOLOR.red) / 65535;
      RGB_component.green = ((float)PXKCOLOR.green) / 65535;
      RGB_component.blue = ((float)PXKCOLOR.blue) / 65535;

      PXKALLOC = YES;
    }

  return PXKCOLOR;
}

- (void)setXColor:(XColor)xcolor
{
  PXKCOLOR = xcolor;
}

@end

@implementation PXKColor

- init
{
  // Allocate backend structure
  be_color_reserved = malloc(sizeof(PXKColor_struct));
  PXKALLOC = NO;

  [super init];

  return self;
}

- (void)dealloc
{
  // Release backend structure
  free(be_color_reserved);

  [super dealloc];
}

- (void)set
{
  NSDPSContext *cur = [NSDPSContext currentContext];
  Display *xdis = [(PXKDPSContext *)cur xDisplay];
  GC xgc = [(PXKDPSContext *)cur xGC];

  if (!PXKALLOC)
    [self xColor];

  // Set the foreground to the color's pixel value
  XSetForeground(xdis, xgc, PXKCOLOR.pixel);

  // Put the GC back into the current context
  [(PXKDPSContext *)cur setGC: xgc];
}

@end


